/***************************************************************************
****************************************************************************

This is a part of the xModule Source code.

Copyright (C) 2003-2004 The LS-Universe Team

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

****************************************************************************
****************************************************************************/

#include "StdAfx.h"
#include "xFont.h"
#include "HTMLDrawer.h"
#include "HTMLFont.h"

xFont::xFont()
{
}

xFont::~xFont()
{
}

void xFont::configure(const string &prefix, xFont *baseFont)
{
	NameValuePair CharSetConstants[] =
	{
		{ "ANSI_CHARSET", ANSI_CHARSET },
		{ "BALTIC_CHARSET", BALTIC_CHARSET },
		{ "CHINESEBIG5_CHARSET", CHINESEBIG5_CHARSET },
		{ "DEFAULT_CHARSET", DEFAULT_CHARSET },
		{ "EASTEUROPE_CHARSET", EASTEUROPE_CHARSET },
		{ "GB2312_CHARSET", GB2312_CHARSET },
		{ "GREEK_CHARSET", GREEK_CHARSET },
		{ "HANGUL_CHARSET", HANGUL_CHARSET },
		{ "MAC_CHARSET", MAC_CHARSET },
		{ "RUSSIAN_CHARSET", RUSSIAN_CHARSET },
		{ "SHIFTJIS_CHARSET", SHIFTJIS_CHARSET },
		{ "SYMBOL_CHARSET", SYMBOL_CHARSET },
		{ "TURKISH_CHARSET", TURKISH_CHARSET },
		{ "VIETNAMESE_CHARSET", VIETNAMESE_CHARSET },
		{ "JOHAB_CHARSET", JOHAB_CHARSET },
		{ "ARABIC_CHARSET", ARABIC_CHARSET },
		{ "HEBREW_CHARSET", HEBREW_CHARSET },
		{ "THAI_CHARSET", THAI_CHARSET },
		{ "OEM_CHARSET", OEM_CHARSET },
		{ 0, 0 }
	};

	shadow = GetxRCBool(prefix, "FontShadow", baseFont ? baseFont->shadow : false);
	shadowColor = GetxRCColor(prefix, "FontShadowColor", baseFont ? baseFont->shadowColor : RGB(128, 128, 128));
	shadowX = GetxRCInt(prefix, "FontShadowX", baseFont ? baseFont->shadowX : 1);
	shadowY = GetxRCInt(prefix, "FontShadowY", baseFont ? baseFont->shadowY : 1);

	textoutline = GetxRCBool(prefix, "FontOutline", baseFont ? baseFont->textoutline : false);
	textoutlineColor = GetxRCColor(prefix, "FontOutlineColor", baseFont ? baseFont->textoutlineColor : RGB(128, 128, 128));

	name = GetxRCString(prefix, "Font", baseFont ? baseFont->name : "Arial");
	charSet = GetxRCNamedValue(prefix, "FontCharSet", CharSetConstants, baseFont ? baseFont->charSet : DEFAULT_CHARSET );
	height = GetxRCInt(prefix, "FontHeight", baseFont ? baseFont->height : 15, 0);
	color = GetxRCColor(prefix, "FontColor", baseFont ? baseFont->color : RGB(0, 0, 0));

	bold = GetxRCBool(prefix, "FontBold", baseFont ? baseFont->bold : false);
	italic = GetxRCBool(prefix, "FontItalic", baseFont ? baseFont->italic : false);
	underline = GetxRCBool(prefix, "FontUnderline", baseFont ? baseFont->underline : false);

	smooth = GetxRCBool(prefix, "FontSmoothing", baseFont ? baseFont->smooth : true);
	if ( IsOS( OS_XP ) && smooth )
		cleartype = GetxRCBool(prefix, "FontClearType", baseFont ? baseFont->cleartype : true);
	else
		cleartype = false;
	lineBreak = GetxRCBool(prefix, "AutoLineBreak", baseFont ? baseFont->lineBreak : false);

	newlineSpace = GetxRCInt(prefix, "NewLineSpace", baseFont ? baseFont->newlineSpace : 0);
}

void xFont::apply(HDC hDC, int left, int right, int top, int bottom, const string &text, int align, int vertAlign, StringList &labelHREFClickRegions)
{
	//Convert into MFC variables
	CDC tmpCDC;
	tmpCDC.Attach(hDC);

	CString strText = text.c_str();
	strText.Replace("<br><br>", "<br> <br>");
	strText.Replace("<br><p>", "<br> <p>");
	strText.Replace("<p><br>", "<p> <br>");
	strText.Replace("<p><p>", "<p> <p>");

	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, name.c_str());
	lf.lfHeight = height;
    lf.lfCharSet = charSet;

	CHTMLFont fntDefault;
	fntDefault.SetBold(bold);
	fntDefault.SetItalic(italic);
	fntDefault.SetUnderline(underline);
	fntDefault.SetSmooth(smooth);
	fntDefault.SetClearType(cleartype);
	fntDefault.SetName(lf.lfFaceName);
	fntDefault.SetCharSet(lf.lfCharSet);
	fntDefault.SetSize(lf.lfHeight);

	short nAlignment = align;
	short nVAlignment = vertAlign;

	CRect rctPosition; 

	// Call to CHTMLDrawer
	if (shadow && !textoutline)
	{
		rctPosition.left = left+shadowX;
		rctPosition.right = right;
		rctPosition.top = top+shadowY;
		rctPosition.bottom = bottom;

		fntDefault.SetColor(shadowColor);

		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								rctPosition,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
	}

	if (textoutline)
	{
		rctPosition.left = left;
		rctPosition.right = right;
		rctPosition.top = top;
		rctPosition.bottom = bottom;

		fntDefault.SetColor(textoutlineColor);

		CRect r;
		CopyRect(&r, &rctPosition);
		r.OffsetRect(1, 1);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
		r.OffsetRect(-1, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
		r.OffsetRect(-1, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
		r.OffsetRect(0, -1);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
		r.OffsetRect(0, -1);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
		r.OffsetRect(1, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
		r.OffsetRect(1, 0);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
		r.OffsetRect(0, 1);
		CHTMLDrawer::DrawText(&tmpCDC,
								strText,
								fntDefault,
								r,
								nAlignment,
								nVAlignment,
								lineBreak,
								newlineSpace,
								labelHREFClickRegions);
	}

	rctPosition.left = left;
	rctPosition.right = right;
	rctPosition.top = top;
	rctPosition.bottom = bottom;

	fntDefault.SetColor(color);

	CHTMLDrawer::DrawText(&tmpCDC,
							strText,
							fntDefault,
							rctPosition,
							nAlignment,
							nVAlignment,
							lineBreak,
							newlineSpace,
							labelHREFClickRegions);

	tmpCDC.Detach();
}

POINT xFont::measure(HDC hDC, int left, int right, const string &text, int align, int vertAlign)
{
	//Convert into MFC variables
	CDC tmpCDC;
	tmpCDC.Attach(hDC);

	CString strText = text.c_str();
	strText.Replace("<br><br>", "<br> <br>");
	strText.Replace("<br><p>", "<br> <p>");
	strText.Replace("<p><br>", "<p> <br>");
	strText.Replace("<p><p>", "<p> <p>");

	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, name.c_str());
	lf.lfHeight = height;
    lf.lfCharSet = charSet;

	CHTMLFont fntDefault;
	fntDefault.SetBold(bold);
	fntDefault.SetItalic(italic);
	fntDefault.SetUnderline(underline);
	fntDefault.SetSmooth(smooth);
	fntDefault.SetClearType(cleartype);
	fntDefault.SetName(lf.lfFaceName);
	fntDefault.SetCharSet(lf.lfCharSet);
	fntDefault.SetSize(lf.lfHeight);

	short nAlignment = align;
	short nVAlignment = vertAlign;

	CRect rctPosition; 
	if (lineBreak)
	{
		rctPosition.left = left;
		rctPosition.right = right;
	}
	else
	{
		rctPosition.left = 0;
		rctPosition.right = 0;
	}
	rctPosition.top = 0;
	rctPosition.bottom = 0;

	// Call to CHTMLDrawer
	POINT returnValues = CHTMLDrawer::MeasureText(&tmpCDC,
													strText,
													fntDefault,
													rctPosition,
													nAlignment,
													nVAlignment,
													lineBreak,
													newlineSpace);

	tmpCDC.Detach();
	
	return returnValues;
}

void xFont::setColor(int aColor, bool shadow)
{
	if (shadow)
		shadowColor = aColor;
	else
		color = aColor;
}

void xFont::setShadow(bool shadowEnabled)
{
	shadow = shadowEnabled;
}

HFONT xFont::gethFont()
{
	LOGFONT lf;
	memset(&lf, 0, sizeof(LOGFONT));

	StringCchCopy(lf.lfFaceName, LF_FACESIZE, name.c_str());
	lf.lfHeight = height;
	lf.lfItalic = italic ? TRUE : FALSE;
	lf.lfUnderline = underline ? TRUE : FALSE;
	lf.lfWeight = bold ? FW_BOLD : FW_NORMAL;
    lf.lfCharSet = charSet;

	return CreateFontIndirect(&lf);
}